@@ -1,30 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the Qt Linguist of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "messagemodel.h"
 #include "statistics.h"
@@ -89,9 +64,9 @@ void ContextItem::appendToComment(const QString &str)
 
 MessageItem *ContextItem::messageItem(int i) const
 {
-    if (i >= 0 && i < msgItemList.count())
+    if (i >= 0 && i < msgItemList.size())
         return const_cast<MessageItem *>(&msgItemList[i]);
-    Q_ASSERT(i >= 0 && i < msgItemList.count());
+    Q_ASSERT(i >= 0 && i < msgItemList.size());
     return 0;
 }
 
@@ -120,20 +95,20 @@ DataModel::DataModel(QObject *parent)
     m_srcCharsSpc(0),
     m_language(QLocale::Language(-1)),
     m_sourceLanguage(QLocale::Language(-1)),
-    m_country(QLocale::Country(-1)),
-    m_sourceCountry(QLocale::Country(-1))
+    m_territory(QLocale::Territory(-1)),
+    m_sourceTerritory(QLocale::Territory(-1))
 {}
 
 QStringList DataModel::normalizedTranslations(const MessageItem &m) const
 {
-    return Translator::normalizedTranslations(m.message(), m_numerusForms.count());
+    return Translator::normalizedTranslations(m.message(), m_numerusForms.size());
 }
 
 ContextItem *DataModel::contextItem(int context) const
 {
-    if (context >= 0 && context < m_contextList.count())
+    if (context >= 0 && context < m_contextList.size())
         return const_cast<ContextItem *>(&m_contextList[context]);
-    Q_ASSERT(context >= 0 && context < m_contextList.count());
+    Q_ASSERT(context >= 0 && context < m_contextList.size());
     return 0;
 }
 
@@ -146,7 +121,7 @@ MessageItem *DataModel::messageItem(const DataIndex &index) const
 
 ContextItem *DataModel::findContext(const QString &context) const
 {
-    for (int c = 0; c < m_contextList.count(); ++c) {
+    for (int c = 0; c < m_contextList.size(); ++c) {
         ContextItem *ctx = contextItem(c);
         if (ctx->context() == context)
             return ctx;
@@ -207,15 +182,15 @@ bool DataModel::load(const QString &fileName, bool *langGuessed, QWidget *parent
     if (!dupes.byId.isEmpty() || !dupes.byContents.isEmpty()) {
         QString err = tr("<qt>Duplicate messages found in '%1':").arg(fileName.toHtmlEscaped());
         int numdups = 0;
-        for (int i : dupes.byId) {
+        for (auto it = dupes.byId.begin(); it != dupes.byId.end(); ++it) {
             if (++numdups >= 5) {
                 err += tr("<p>[more duplicates omitted]");
                 goto doWarn;
             }
-            err += tr("<p>* ID: %1").arg(tor.message(i).id().toHtmlEscaped());
+            err += tr("<p>* ID: %1").arg(tor.message(it.key()).id().toHtmlEscaped());
         }
-        for (int j : dupes.byContents) {
-            const TranslatorMessage &msg = tor.message(j);
+        for (auto it = dupes.byContents.begin(); it != dupes.byContents.end(); ++it) {
+            const TranslatorMessage &msg = tor.message(it.key());
             if (++numdups >= 5) {
                 err += tr("<p>[more duplicates omitted]");
                 break;
@@ -283,15 +258,15 @@ bool DataModel::load(const QString &fileName, bool *langGuessed, QWidget *parent
         *langGuessed = true;
     }
     QLocale::Language l;
-    QLocale::Country c;
-    Translator::languageAndCountry(lang, &l, &c);
+    QLocale::Territory c;
+    Translator::languageAndTerritory(lang, &l, &c);
     if (l == QLocale::C) {
         QLocale sys;
         l = sys.language();
-        c = sys.country();
+        c = sys.territory();
         *langGuessed = true;
     }
-    if (!setLanguageAndCountry(l, c))
+    if (!setLanguageAndTerritory(l, c))
         QMessageBox::warning(parent, QObject::tr("Qt Linguist"),
                              tr("Linguist does not know the plural rules for '%1'.\n"
                                 "Will assume a single universal form.")
@@ -303,11 +278,11 @@ bool DataModel::load(const QString &fileName, bool *langGuessed, QWidget *parent
     lang = tor.sourceLanguageCode();
     if (lang.isEmpty()) {
         l = QLocale::C;
-        c = QLocale::AnyCountry;
+        c = QLocale::AnyTerritory;
     } else {
-        Translator::languageAndCountry(lang, &l, &c);
+        Translator::languageAndTerritory(lang, &l, &c);
     }
-    setSourceLanguageAndCountry(l, c);
+    setSourceLanguageAndTerritory(l, c);
 
     setModified(false);
 
@@ -320,8 +295,8 @@ bool DataModel::save(const QString &fileName, QWidget *parent)
     for (DataModelIterator it(this); it.isValid(); ++it)
         tor.append(it.current()->message());
 
-    tor.setLanguageCode(Translator::makeLanguageCode(m_language, m_country));
-    tor.setSourceLanguageCode(Translator::makeLanguageCode(m_sourceLanguage, m_sourceCountry));
+    tor.setLanguageCode(Translator::makeLanguageCode(m_language, m_territory));
+    tor.setSourceLanguageCode(Translator::makeLanguageCode(m_sourceLanguage, m_sourceTerritory));
     tor.setLocationsType(m_relativeLocations ? Translator::RelativeLocations
                                              : Translator::AbsoluteLocations);
     tor.setExtras(m_extra);
@@ -353,7 +328,7 @@ bool DataModel::release(const QString &fileName, bool verbose, bool ignoreUnfini
         return false;
     }
     Translator tor;
-    QLocale locale(m_language, m_country);
+    QLocale locale(m_language, m_territory);
     tor.setLanguageCode(locale.name());
     for (DataModelIterator it(this); it.isValid(); ++it)
         tor.append(it.current()->message());
@@ -361,6 +336,9 @@ bool DataModel::release(const QString &fileName, bool verbose, bool ignoreUnfini
     cd.m_verbose = verbose;
     cd.m_ignoreUnfinished = ignoreUnfinished;
     cd.m_saveMode = mode;
+    cd.m_idBased =
+            std::all_of(tor.messages().begin(), tor.messages().end(),
+                        [](const TranslatorMessage &message) { return !message.id().isEmpty(); });
     bool ok = saveQM(tor, file, cd);
     if (!ok)
         QMessageBox::warning(parent, QObject::tr("Qt Linguist"), cd.error());
@@ -369,9 +347,9 @@ bool DataModel::release(const QString &fileName, bool verbose, bool ignoreUnfini
 
 void DataModel::doCharCounting(const QString &text, int &trW, int &trC, int &trCS)
 {
-    trCS += text.length();
+    trCS += text.size();
     bool inWord = false;
-    for (int i = 0; i < text.length(); ++i) {
+    for (int i = 0; i < text.size(); ++i) {
         if (text[i].isLetterOrNumber() || text[i] == QLatin1Char('_')) {
             if (!inWord) {
                 ++trW;
@@ -385,21 +363,30 @@ void DataModel::doCharCounting(const QString &text, int &trW, int &trC, int &trC
     }
 }
 
-bool DataModel::setLanguageAndCountry(QLocale::Language lang, QLocale::Country country)
+bool DataModel::setLanguageAndTerritory(QLocale::Language lang, QLocale::Territory territory)
 {
-    if (m_language == lang && m_country == country)
+    if (m_language == lang && m_territory == territory)
         return true;
     m_language = lang;
-    m_country = country;
+    m_territory = territory;
 
     if (lang == QLocale::C || uint(lang) > uint(QLocale::LastLanguage)) // XXX does this make any sense?
         lang = QLocale::English;
     QByteArray rules;
-    bool ok = getNumerusInfo(lang, country, &rules, &m_numerusForms, 0);
-    QLocale loc(lang, country);
-    m_localizedLanguage = QLocale::countriesForLanguage(lang).size() > 1
-            //: <language> (<country>)
-            ? tr("%1 (%2)").arg(loc.nativeLanguageName(), loc.nativeCountryName())
+    bool ok = getNumerusInfo(lang, territory, &rules, &m_numerusForms, 0);
+    QLocale loc(lang, territory);
+    // Add territory name if we couldn't match the (lang, territory) combination,
+    // or if the language is used in more than one territory.
+    const bool mentionTerritory = (loc.territory() != territory) || [lang, territory]() {
+        const auto locales = QLocale::matchingLocales(lang, QLocale::AnyScript,
+                                                      QLocale::AnyTerritory);
+        return std::any_of(locales.cbegin(), locales.cend(), [territory](const QLocale &locale) {
+            return locale.territory() != territory;
+        });
+    }();
+    m_localizedLanguage = mentionTerritory
+            //: <language> (<territory>)
+            ? tr("%1 (%2)").arg(loc.nativeLanguageName(), loc.nativeTerritoryName())
             : loc.nativeLanguageName();
     m_countRefNeeds.clear();
     for (int i = 0; i < rules.size(); ++i) {
@@ -416,12 +403,12 @@ bool DataModel::setLanguageAndCountry(QLocale::Language lang, QLocale::Country c
     return ok;
 }
 
-void DataModel::setSourceLanguageAndCountry(QLocale::Language lang, QLocale::Country country)
+void DataModel::setSourceLanguageAndTerritory(QLocale::Language lang, QLocale::Territory territory)
 {
-    if (m_sourceLanguage == lang && m_sourceCountry == country)
+    if (m_sourceLanguage == lang && m_sourceTerritory == territory)
         return;
     m_sourceLanguage = lang;
-    m_sourceCountry = country;
+    m_sourceTerritory = territory;
     setModified(true);
 }
 
@@ -472,7 +459,7 @@ QString DataModel::prettifyPlainFileName(const QString &fn)
 {
     static QString workdir = QDir::currentPath() + QLatin1Char('/');
 
-    return QDir::toNativeSeparators(fn.startsWith(workdir) ? fn.mid(workdir.length()) : fn);
+    return QDir::toNativeSeparators(fn.startsWith(workdir) ? fn.mid(workdir.size()) : fn);
 }
 
 QString DataModel::prettifyFileName(const QString &fn)
@@ -496,7 +483,7 @@ DataModelIterator::DataModelIterator(DataModel *model, int context, int message)
 
 bool DataModelIterator::isValid() const
 {
-    return m_context < m_model->m_contextList.count();
+    return m_context < m_model->m_contextList.size();
 }
 
 void DataModelIterator::operator++()
@@ -604,9 +591,9 @@ void MultiContextItem::putMessageItem(int pos, MessageItem *m)
 void MultiContextItem::appendMessageItems(const QList<MessageItem *> &m)
 {
     QList<MessageItem *> nullItems = m; // Basically, just a reservation
-    for (int i = 0; i < nullItems.count(); ++i)
+    for (int i = 0; i < nullItems.size(); ++i)
         nullItems[i] = 0;
-    for (int i = 0; i < m_messageLists.count() - 1; ++i)
+    for (int i = 0; i < m_messageLists.size() - 1; ++i)
         m_messageLists[i] += nullItems;
     m_messageLists.last() += m;
     for (MessageItem *mi : m)
@@ -615,7 +602,7 @@ void MultiContextItem::appendMessageItems(const QList<MessageItem *> &m)
 
 void MultiContextItem::removeMultiMessageItem(int pos)
 {
-    for (int i = 0; i < m_messageLists.count(); ++i)
+    for (int i = 0; i < m_messageLists.size(); ++i)
         m_messageLists[i].removeAt(pos);
     m_multiMessageList.removeAt(pos);
 }
@@ -798,7 +785,7 @@ void MultiDataModel::append(DataModel *dm, bool readWrite)
 
 void MultiDataModel::close(int model)
 {
-    if (m_dataModels.count() == 1) {
+    if (m_dataModels.size() == 1) {
         closeAll();
     } else {
         updateCountsOnRemove(model, isModelWritable(model));
@@ -870,34 +857,34 @@ QString MultiDataModel::condenseFileNames(const QStringList &names)
     if (names.isEmpty())
         return QString();
 
-    if (names.count() < 2)
+    if (names.size() < 2)
         return names.first();
 
     QString prefix = names.first();
     if (prefix.startsWith(QLatin1Char('=')))
         prefix.remove(0, 1);
     QString suffix = prefix;
-    for (int i = 1; i < names.count(); ++i) {
+    for (int i = 1; i < names.size(); ++i) {
         QString fn = names[i];
         if (fn.startsWith(QLatin1Char('=')))
             fn.remove(0, 1);
-        for (int j = 0; j < prefix.length(); ++j)
+        for (int j = 0; j < prefix.size(); ++j)
             if (fn[j] != prefix[j]) {
-                if (j < prefix.length()) {
+                if (j < prefix.size()) {
                     while (j > 0 && prefix[j - 1].isLetterOrNumber())
                         --j;
                     prefix.truncate(j);
                 }
                 break;
             }
-        int fnl = fn.length() - 1;
-        int sxl = suffix.length() - 1;
+        int fnl = fn.size() - 1;
+        int sxl = suffix.size() - 1;
         for (int k = 0; k <= sxl; ++k)
             if (fn[fnl - k] != suffix[sxl - k]) {
                 if (k < sxl) {
                     while (k > 0 && suffix[sxl - k + 1].isLetterOrNumber())
                         --k;
-                    if (prefix.length() + k > fnl)
+                    if (prefix.size() + k > fnl)
                         --k;
                     suffix.remove(0, sxl - k + 1);
                 }
@@ -905,9 +892,9 @@ QString MultiDataModel::condenseFileNames(const QStringList &names)
             }
     }
     QString ret = prefix + QLatin1Char('{');
-    int pxl = prefix.length();
-    int sxl = suffix.length();
-    for (int j = 0; j < names.count(); ++j) {
+    int pxl = prefix.size();
+    int sxl = suffix.size();
+    for (int j = 0; j < names.size(); ++j) {
         if (j)
             ret += QLatin1Char(',');
         int off = pxl;
@@ -916,7 +903,7 @@ QString MultiDataModel::condenseFileNames(const QStringList &names)
             ret += QLatin1Char('=');
             ++off;
         }
-        ret += fn.mid(off, fn.length() - sxl - off);
+        ret += fn.mid(off, fn.size() - sxl - off);
     }
     ret += QLatin1Char('}') + suffix;
     return ret;
@@ -1184,7 +1171,7 @@ void MultiDataModelIterator::operator++()
 
 bool MultiDataModelIterator::isValid() const
 {
-    return m_context < m_dataModel->m_multiContextList.count();
+    return m_context < m_dataModel->m_multiContextList.size();
 }
 
 MessageItem *MultiDataModelIterator::current() const
